/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.cassandra.tcm.listeners;

import java.util.EnumSet;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.apache.cassandra.tcm.ClusterMetadata;
import org.apache.cassandra.tcm.ClusterMetadataService;
import org.apache.cassandra.tcm.Transformation;
import org.apache.cassandra.tcm.log.Entry;

import static org.apache.cassandra.tcm.Transformation.Kind.FORCE_SNAPSHOT;
import static org.apache.cassandra.tcm.Transformation.Kind.TRIGGER_SNAPSHOT;

public class MetadataSnapshotListener implements LogListener
{
    private static final Logger logger = LoggerFactory.getLogger(MetadataSnapshotListener.class);

    private static final EnumSet<Transformation.Kind> triggers = EnumSet.of(TRIGGER_SNAPSHOT, FORCE_SNAPSHOT);

    @Override
    public void notify(Entry entry, Transformation.Result result)
    {
        ClusterMetadata next = result.success().metadata;
        if (triggers.contains(entry.transform.kind()))
        {
            try
            {
                ClusterMetadataService.instance().snapshotManager().storeSnapshot(next);
            }
            catch (Throwable e)
            {
                logger.warn("Unable to serialize metadata snapshot triggered by TriggerSnapshot transformation", e);
            }
        }
    }
}
